@@ -15,7 +15,7 @@
 ** or VDBE.  The VDBE implements an abstract machine that runs a
 ** simple program to access and modify the underlying database.
 **
-** $Id: vdbe.h,v 1.2 2002/02/27 19:25:23 matt Exp $
+** $Id: vdbe.h,v 1.3 2002/03/12 15:43:02 matt Exp $
 */
 #ifndef _SQLITE_VDBE_H_
 #define _SQLITE_VDBE_H_
@@ -97,89 +97,89 @@ typedef struct VdbeOp VdbeOp;
 #define OP_KeyAsData          25
 #define OP_Recno              26
 #define OP_FullKey            27
-#define OP_Rewind             28
-#define OP_Next               29
+#define OP_NullRow            28
+#define OP_Last               29
+#define OP_Rewind             30
+#define OP_Next               31
 
-#define OP_Destroy            30
-#define OP_Clear              31
-#define OP_CreateIndex        32
-#define OP_CreateTable        33
-#define OP_SanityCheck        34
+#define OP_Destroy            32
+#define OP_Clear              33
+#define OP_CreateIndex        34
+#define OP_CreateTable        35
+#define OP_IntegrityCk        36
 
-#define OP_IdxPut             35
-#define OP_IdxDelete          36
-#define OP_IdxRecno           37
-#define OP_IdxGT              38
-#define OP_IdxGE              39
+#define OP_IdxPut             37
+#define OP_IdxDelete          38
+#define OP_IdxRecno           39
+#define OP_IdxGT              40
+#define OP_IdxGE              41
 
-#define OP_MemLoad            40
-#define OP_MemStore           41
+#define OP_MemLoad            42
+#define OP_MemStore           43
 
-#define OP_ListWrite          42
-#define OP_ListRewind         43
-#define OP_ListRead           44
-#define OP_ListReset          45
+#define OP_ListWrite          44
+#define OP_ListRewind         45
+#define OP_ListRead           46
+#define OP_ListReset          47
 
-#define OP_SortPut            46
-#define OP_SortMakeRec        47
-#define OP_SortMakeKey        48
-#define OP_Sort               49
-#define OP_SortNext           50
-#define OP_SortCallback       51
-#define OP_SortReset          52
+#define OP_SortPut            48
+#define OP_SortMakeRec        49
+#define OP_SortMakeKey        50
+#define OP_Sort               51
+#define OP_SortNext           52
+#define OP_SortCallback       53
+#define OP_SortReset          54
 
-#define OP_FileOpen           53
-#define OP_FileRead           54
-#define OP_FileColumn         55
+#define OP_FileOpen           55
+#define OP_FileRead           56
+#define OP_FileColumn         57
 
-#define OP_AggReset           56
-#define OP_AggFocus           57
-#define OP_AggIncr            58
-#define OP_AggNext            59
-#define OP_AggSet             60
-#define OP_AggGet             61
+#define OP_AggReset           58
+#define OP_AggFocus           59
+#define OP_AggNext            60
+#define OP_AggSet             61
+#define OP_AggGet             62
+#define OP_AggFunc            63
+#define OP_AggInit            64
 
-#define OP_SetInsert          62
-#define OP_SetFound           63
-#define OP_SetNotFound        64
+#define OP_SetInsert          65
+#define OP_SetFound           66
+#define OP_SetNotFound        67
 
-#define OP_MakeRecord         65
-#define OP_MakeKey            66
-#define OP_MakeIdxKey         67
-#define OP_IncrKey            68
+#define OP_MakeRecord         68
+#define OP_MakeKey            69
+#define OP_MakeIdxKey         70
+#define OP_IncrKey            71
 
-#define OP_Goto               69
-#define OP_If                 70
-#define OP_Halt               71
+#define OP_Goto               72
+#define OP_If                 73
+#define OP_Halt               74
 
-#define OP_ColumnCount        72
-#define OP_ColumnName         73
-#define OP_Callback           74
-#define OP_NullCallback       75
+#define OP_ColumnCount        75
+#define OP_ColumnName         76
+#define OP_Callback           77
+#define OP_NullCallback       78
 
-#define OP_Integer            76
-#define OP_String             77
-#define OP_Pop                78
-#define OP_Dup                79
-#define OP_Pull               80
-#define OP_Push               81
-#define OP_MustBeInt          82
+#define OP_Integer            79
+#define OP_String             80
+#define OP_Pop                81
+#define OP_Dup                82
+#define OP_Pull               83
+#define OP_Push               84
+#define OP_MustBeInt          85
 
-#define OP_Add                83
-#define OP_AddImm             84
-#define OP_Subtract           85
-#define OP_Multiply           86
-#define OP_Divide             87
-#define OP_Remainder          88
-#define OP_BitAnd             89
-#define OP_BitOr              90
-#define OP_BitNot             91
-#define OP_ShiftLeft          92
-#define OP_ShiftRight         93
-#define OP_AbsValue           94
-#define OP_Precision          95
-#define OP_Min                96
-#define OP_Max                97
+#define OP_Add                86
+#define OP_AddImm             87
+#define OP_Subtract           88
+#define OP_Multiply           89
+#define OP_Divide             90
+#define OP_Remainder          91
+#define OP_BitAnd             92
+#define OP_BitOr              93
+#define OP_BitNot             94
+#define OP_ShiftLeft          95
+#define OP_ShiftRight         96
+#define OP_AbsValue           97
 #define OP_Like               98
 #define OP_Glob               99
 #define OP_Eq                100
@@ -196,13 +196,11 @@ typedef struct VdbeOp VdbeOp;
 #define OP_Not               111
 #define OP_Concat            112
 #define OP_Noop              113
+#define OP_Function          114
 
-#define OP_Strlen            114
-#define OP_Substr            115
+#define OP_Limit             115
 
-#define OP_Limit             116
-
-#define OP_MAX               116
+#define OP_MAX               115
 
 /*
 ** Prototypes for the VDBE interface.  See comments on the implementation
